@@ -196,10 +196,6 @@ def setup_environment_listeners(context: Context, tasks: List[GrizzlyTask]) -> S
     # initialize testdata
     try:
         testdata, external_dependencies = initialize_testdata(grizzly, tasks)
-
-        for scenario_testdata in testdata.values():
-            for variable, value in scenario_testdata.items():
-                assert value is not None, f'variable {variable} has not been initialized'
     except TemplateError as e:
         logger.error(e, exc_info=True)
         raise AssertionError(f'error parsing request payload: {e}') from e
@@ -469,6 +465,7 @@ def run(context: Context) -> int:
                         count = 0
 
                 logger.info(f'{runner.user_count=}, quit {runner.__class__.__name__}')
+                runner.environment.events.quitting.fire(environment=runner.environment, reverse=True)
                 if isinstance(runner, MasterRunner):
                     runner.send_message('grizzly_worker_quit', None)
 
@@ -520,6 +517,7 @@ def run(context: Context) -> int:
 
         def sig_term_handler() -> None:
             logger.info('got SIGTERM signal')
+            runner.environment.events.quitting.fire(environment=runner.environment, reverse=True)
             runner.quit()
 
         gevent.signal_handler(SIGTERM, sig_term_handler)
