@@ -42,8 +42,8 @@ The MIME type of an uploaded file will automagically be guessed based on the [re
 '''
 import logging
 
-from typing import Any, Optional, cast
-from urllib.parse import urlparse, parse_qs
+from typing import Optional, List, cast
+from urllib.parse import urlparse, parse_qs, quote
 from pathlib import Path
 from mimetypes import guess_type as mimetype_guess
 
@@ -53,8 +53,7 @@ from azure.storage.blob import BlobServiceClient, ContentSettings
 from . import client, ClientTask
 from ...scenarios import GrizzlyScenario
 from ...context import GrizzlyContextScenario
-from ...types import RequestDirection
-from ...testdata.utils import resolve_variable
+from ...types import RequestDirection, GrizzlyResponse
 
 # disable verbose INFO logging
 azure_logger = logging.getLogger('azure.core.pipeline.policies.http_logging_policy')
@@ -85,12 +84,20 @@ class BlobStorageClientTask(ClientTask):
 
         parsed = urlparse(self.endpoint)
 
-        self._endpoints_protocol = 'http' if parsed.scheme == 'bs' else 'https'
+        self._endpoints_protocol = 'http' if self._scheme == 'bs' else 'https'
 
         if parsed.netloc is None or len(parsed.netloc) < 1:
             raise ValueError(f'{self.__class__.__name__}: could not find account name in {self.endpoint}')
 
-        parameters = parse_qs(parsed.query)
+        # See urllib/parse.py:771-774, explicit + characters are replaced with white space,
+        # AccountKey could contain explicit + characters, so we must quote them first.
+        parsed_query: List[str] = []
+        if len(parsed.query) > 0:
+            for parameter in parsed.query.split('&'):
+                key, value = parameter.split('=', 1)
+                parsed_query.append(f'{quote(key)}={quote(value)}')
+
+        parameters = parse_qs('&'.join(parsed_query))
 
         if 'AccountKey' not in parameters:
             raise ValueError(f'{self.__class__.__name__}: could not find AccountKey in {self.endpoint}')
@@ -98,9 +105,9 @@ class BlobStorageClientTask(ClientTask):
         if 'Container' not in parameters:
             raise ValueError(f'{self.__class__.__name__}: could not find Container in {self.endpoint}')
 
-        self.account_name = cast(str, resolve_variable(self.grizzly, parsed.netloc, guess_datatype=False))
-        self.account_key = cast(str, resolve_variable(self.grizzly, parameters['AccountKey'][0], guess_datatype=False))
-        self.container = cast(str, resolve_variable(self.grizzly, parameters['Container'][0], guess_datatype=False))
+        self.account_name = parsed.netloc
+        self.account_key = parameters['AccountKey'][0]
+        self.container = parameters['Container'][0]
 
         self.service_client = BlobServiceClient.from_connection_string(conn_str=self.connection_string)
 
@@ -108,10 +115,10 @@ class BlobStorageClientTask(ClientTask):
     def connection_string(self) -> str:
         return f'DefaultEndpointsProtocol={self._endpoints_protocol};AccountName={self.account_name};AccountKey={self.account_key};EndpointSuffix=core.windows.net'
 
-    def get(self, parent: GrizzlyScenario) -> Any:
+    def get(self, parent: GrizzlyScenario) -> GrizzlyResponse:
         return super().get(parent)
 
-    def put(self, parent: GrizzlyScenario) -> Any:
+    def put(self, parent: GrizzlyScenario) -> GrizzlyResponse:
         source = parent.render(cast(str, self.source))
 
         if self.destination is not None:
@@ -137,3 +144,5 @@ class BlobStorageClientTask(ClientTask):
             with self.service_client.get_blob_client(container=self.container, blob=destination) as blob_client:
                 blob_client.upload_blob(source, content_settings=content_settings)
                 meta['response_length'] = len(source)
+
+        return None, None
