@@ -1,4 +1,6 @@
+import itertools
 import logging
+import re
 
 from typing import TYPE_CHECKING, Optional, List, Dict, Any, Tuple, Set
 from collections import namedtuple
@@ -27,6 +29,31 @@ def initialize_testdata(grizzly: 'GrizzlyContext', tasks: List[GrizzlyTask]) ->
     testdata: TestdataType = {}
     template_variables = get_template_variables(tasks)
 
+    found_variables = set()
+    for variable in itertools.chain(*template_variables.values()):
+        module_name, variable_type, variable_name = GrizzlyVariables.get_variable_spec(variable)
+
+        if module_name is None and variable_type is None:
+            found_variables.add(variable_name)
+        else:
+            prefix = f'{module_name}.' if module_name != 'grizzly.testdata.variables' else ''
+            found_variables.add(f'{prefix}{variable_type}.{variable_name}')
+
+    declared_variables = set(grizzly.state.variables.keys())
+
+    # check except between declared variables and variables found in templates
+    missing_in_templates = []
+    for variable in declared_variables:
+        if variable not in found_variables:
+            missing_in_templates.append(variable)
+    assert len(missing_in_templates) == 0, f'variables has been declared, but cannot be found in templates: {",".join(missing_in_templates)}'
+
+    missing_declarations = []
+    for variable in found_variables:
+        if variable not in declared_variables:
+            missing_declarations.append(variable)
+    assert len(missing_declarations) == 0, f'variables has been found in templates, but have not been declared: {",".join(missing_declarations)}'
+
     initialized_datatypes: Dict[str, Any] = {}
     external_dependencies: Set[str] = set()
 
@@ -119,6 +146,9 @@ def _objectify(testdata: Dict[str, Any]) -> Dict[str, Any]:
 
 
 def create_context_variable(grizzly: 'GrizzlyContext', variable: str, value: str) -> Dict[str, Any]:
+    if '{{' in value and '}}' in value:
+        grizzly.scenario.orphan_templates.append(value)
+
     casted_value = resolve_variable(grizzly, value)
 
     variable = variable.lower().replace(' ', '_').replace('/', '.')
@@ -126,7 +156,7 @@ def create_context_variable(grizzly: 'GrizzlyContext', variable: str, value: str
     return transform(grizzly, {variable: casted_value}, objectify=False)
 
 
-def resolve_variable(grizzly: 'GrizzlyContext', value: str, guess_datatype: Optional[bool] = True) -> GrizzlyVariableType:
+def resolve_variable(grizzly: 'GrizzlyContext', value: str, guess_datatype: Optional[bool] = True, only_grizzly: bool = False) -> GrizzlyVariableType:
     if len(value) < 1:
         return value
 
@@ -136,7 +166,7 @@ def resolve_variable(grizzly: 'GrizzlyContext', value: str, guess_datatype: Opti
         value = value[1:-1]
 
     resolved_variable: GrizzlyVariableType
-    if '{{' in value and '}}' in value:
+    if '{{' in value and '}}' in value and not only_grizzly:
         template = Template(value)
         j2env = Environment(autoescape=False)
         template_parsed = j2env.parse(value)
@@ -146,18 +176,34 @@ def resolve_variable(grizzly: 'GrizzlyContext', value: str, guess_datatype: Opti
             assert template_variable in grizzly.state.variables, f'value contained variable "{template_variable}" which has not been set'
 
         resolved_variable = template.render(**grizzly.state.variables)
-    elif len(value) > 4 and value[0] == '$' and value[1] != '.':  # $. is jsonpath expression...
-        if value[:5] == '$conf':
-            variable = value[7:]
-            assert variable in grizzly.state.configuration, f'configuration variable "{variable}" is not set'
-            resolved_variable = grizzly.state.configuration[variable]
-        elif value[:4] == '$env':
-            variable = value[6:]
-            env_value = environ.get(variable, None)
-            assert env_value is not None, f'environment variable "{variable}" is not set'
-            resolved_variable = env_value
-        else:
-            raise ValueError(f'{value.split("::", 1)[0]} is not implemented')
+    elif '$conf' in value or '$env' in value:
+        regex = r"\$(conf|env)::([^\$]+)\$"
+
+        matches = re.finditer(regex, value, re.MULTILINE)
+
+        if matches:
+            resolved_variable = value
+
+        for match in matches:
+            match_type = match.group(1)
+            variable_name = match.group(2)
+
+            if match_type == 'conf':
+                assert variable_name in grizzly.state.configuration, f'configuration variable "{variable_name}" is not set'
+                variable_value = grizzly.state.configuration[variable_name]
+            elif match_type == 'env':
+                assert variable_name in environ, f'environment variable "{variable_name}" is not set'
+                variable_value = environ.get(variable_name, None)
+
+            if not isinstance(variable_value, str):
+                variable_value = str(variable_value)
+
+            value = value.replace(f'${match_type}::{variable_name}$', variable_value)
+
+        resolved_variable = value
+
+        if len(value) > 4 and value[0] == '$' and value[1] != '.':  # $. is jsonpath expression...
+            raise ValueError(f'"{value}" is not a correctly specified templating variable, variables must match "$(conf|env)::<variable name>$"')
     else:
         resolved_variable = value
 
